
// C++ template program to soolve the Damped-Driven Pendulum
// via the second-order predictor-corrector (Verlet) scheme.

#include <iostream>
#include <stdlib.h>
#include <cmath>
using namespace std;

// Global parameters.

const double X0 = 0;       // initial angle
const double V0 = 0.1;     // initial angular velocity
const double K  = 1.0;     // pendulum constant  (  g/l  )
const double DT = 0.001;    // time dt
const double TMAX = 20;    // time domain
const double mass = 1.0;   // mass
double alpha = 0.0;        // dissipative force
double g = 0.0;            // driving force amplitude
double Omega = 1.0;        // and frequency
double Transients = 0;     // transients
int mPIpPI = 0;            // projection into [-PI,PI]

                           // To project on a stroboscopic SoS   pipe
                           // all the points (mPIpPI=0) into the
                           // spectroscoptc filter 

//-------------------------------------- modify only above this line

double acc(double x, double v, double t)	// acceleration
{
    return ( -K*sin(x) - alpha*v + g*cos(Omega*t) ) /mass;
}

double potential(double x)			// keep consistent with acc!
{
    return K*(1-cos(x));
}

double energy(double x, double v)
{
    return potential(x) + 0.5*mass*v*v;
}

//-------------------------------------------------------------------------

double E0;
void output(double x, double v, double t)
{
    cout << t << " " << x << " " << v << " " << energy(x, v) - E0 << endl;
}

bool time_to_stop(double x, double v, double t, double tmax)
{
    // Stopping condition.

    return (t > tmax);
}

void take_a_step(double &x, double &v, double &t, double dt)
{
    // Take a single time step.

    double a = acc(x, v, t);

    // Prediction:

    x += v*dt + 0.5*a*dt*dt;
    v += a*dt;
    t += dt;

    // Correction:

    double a1 = acc(x, v, t);
    v += 0.5*dt*(a1-a);
}

int main(int argc, char *argv[])
{
    // Declaration and initialization.

    double t = 0, x = X0, v = V0;
    double tmax = TMAX, dt = DT;

    if (argc > 1)
	v = atof(argv[1]);	// first command-line argument sets V0
    if (argc > 2)
	tmax = atof(argv[2]);	// second command-line argument sets tmax
    if ( argc > 3 )
        alpha = atof(argv[3]);
    if ( argc > 4 ) {
         g = atof(argv[4]);
         Omega = atof(argv[5]);
            }
    if ( argc > 5 )
        Transients = atof(argv[6]);
    if ( argc > 6  )
        mPIpPI = atoi(argv[7]);

    cerr << " v  " << v << " tmax " << tmax ;
    cerr << " alpha " << alpha << " g " << g << " Omega " << Omega << endl;
    cerr << " Transients " << Transients << " mPIpPI " << mPIpPI << endl;

    E0 = energy(x, v);	        // initial energy; global for convenience
    output(x, v, t);

    // Integrate the motion.

    while (!time_to_stop(x, v, t, tmax)) {
                                               // ODE integration
	take_a_step(x, v, t, dt);
                                               // restric to -PI -- PI range
        if ( mPIpPI == 1 )
          {
             while ( x > M_PI ) x = x - 2 * M_PI;
             while ( x < - M_PI ) x = x + 2 * M_PI;
          }
                                               // store trajs
        if ( t > Transients )                  // remove transients
                output(x, v, t);
    }
}








